package rangesearch;

import rangesearch.api.*;
import rangesearch.ref.*;

import java.io.*;
import jdsl.core.algo.sorts.ArrayQuickSort;
import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;

/** Implements features which are not in JDSL. */
public class JDSLUtil {
    static protected final int TYPE_NULL = 0;
    static protected final int TYPE_INT  = 1;
    static protected final int TYPE_RAT  = 2;

    /** Converts an iterator over 2D points into a string.
     * @return a string representation of the list of points, lexographically
     * sorted from left to right, and from bottom to top in the case of a tie.
     * */
    static public String pointListToString(Point2DIterator pi) {
        Sequence a = new ArraySequence();
        while(pi.hasNext()) {
            Point2D p = pi.nextPoint2D();
            a.insertLast(p);
        }
        GeomTester2D tester = new GeomTester2DImpl();
        LexographicComparator lc = new LexographicComparator(tester);
        ArrayQuickSort quickSort = new ArrayQuickSort();
        quickSort.sort(a, lc);
        
        ObjectIterator oi = a.elements();
        String s = "[ ";
        while(oi.hasNext()) {
            Point2D p = (Point2D) oi.nextObject();
            s = s + pointToString(p) + " ";
        }
        s = s + "]";
        return s;
    }

    /** Converts a point to a string representation.
     * @return the string representation. */
    static public String pointToString(Point2D p) {
        String s = "(";
        if(isInt(p.x()) && isInt(p.y())) {
            s = s + (int) p.x() + ", " + (int) p.y() + ")";
        } else {
            s = s + p.x() + ", " + p.y() + ")";
        }
        return s;
    }
    static private boolean isInt(double d) {
        return (d == (double) ((int) d));
    }

    /** Writes a point to a text stream.
     * @throws IOException if the point can't be written. */
    static public void savePoint(Writer s, Point2D p)
            throws IOException {
        if(p == null) {
            s.write(TYPE_NULL + " ");
            s.flush();
            return;
        }
        try {
            IntPoint2D ip = (IntPoint2D) p;
            s.write(TYPE_INT + " ");
            s.write(ip.intx() + " ");
            s.write(ip.inty() + " ");
            s.flush();
            return;
        } catch(ClassCastException ex) {
        }

        try {
            RatPoint2D rp = (RatPoint2D) p;
            s.write(TYPE_RAT + " ");
            s.write(rp.numx() + " ");
            s.write(rp.numy() + " ");
            s.write(rp.numw() + " ");
            s.flush();
            return;
        } catch(ClassCastException ex) {
        }

        throw new RuntimeException("Unrecognized point class.");
    }

    /** Reads a point from a text stream.
     * @throws IOException if the point can't be read. */
    static public Point2D loadPoint(StreamTokenizer st)
            throws IOException {
        // XXX - implement me.
        int type = (int) readNumber(st);
        if(type == TYPE_NULL) {
            return null;
        } else if(type == TYPE_INT) {
            long x = (long) readNumber(st);
            long y = (long) readNumber(st);
            return new IntPoint2D(x, y);
        } else if(type == TYPE_RAT) {
            Number x = new Double(readNumber(st));
            Number y = new Double(readNumber(st));
            Number w = new Double(readNumber(st));
            return new RatPoint2D(x, y, w);
        } else {
            throw new RuntimeException("Unrecognized point class.");
        }
    }

    /** Helper function for loadPoint.
     * @return the number read from the stream, in double form.
     * @throws IOException if the number can't be read. */
    static public double readNumber(StreamTokenizer st)
            throws IOException {
        int t = st.nextToken();
        if(t != StreamTokenizer.TT_NUMBER) {
            System.out.println(StreamTokenizer.TT_NUMBER + " " +
                    StreamTokenizer.TT_WORD + " " +
                    t);
            throw new IOException("Invalid format: Expected number, got '" +
                    st.toString() + "' on line " +
                    st.lineno() + ".");
        }
        return st.nval;
    }
}

