package rangesearch;

import rangesearch.api.*;
import rangesearch.ref.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.filechooser.FileFilter;
import java.net.URL;
import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;

/** This class tracks the application state and handles high-level user
 * interaction for the graphical interface. */

public class RangeFrame extends JFrame {
    protected JFileChooser m_fileChooser;
    protected RangeView m_view;
    protected JToolBar m_toolBar;
    protected JToggleButton m_pointButton;
    protected JToggleButton m_rangeButton;
    protected GenOptionsDialog m_genDialog;
    protected int m_genPoints;
    
    public RangeFrame() {
        setTitle("2D range search");
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                exit();
            }
        });

        // Create menubar.
        JMenu menu;
        JMenuItem menuItem;
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        menu = new JMenu("File");
        menu.setMnemonic(KeyEvent.VK_F);
        menuBar.add(menu);

        menuItem = new JMenuItem("New");
        menuItem.setMnemonic(KeyEvent.VK_N);
        menuItem.addActionListener(new FileNewListener());
        menu.add(menuItem);

        menuItem = new JMenuItem("Open...");
        menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.addActionListener(new FileOpenListener());
        menu.add(menuItem);

        menuItem = new JMenuItem("Save...");
        menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.addActionListener(new FileSaveListener());
        menu.add(menuItem);

        menu.addSeparator();

        menuItem = new JMenuItem("Exit");
        menuItem.setMnemonic(KeyEvent.VK_X);
        menuItem.addActionListener(new FileExitListener());
        menu.add(menuItem);

        menu = new JMenu("View");
        menu.setMnemonic(KeyEvent.VK_V);
        menuBar.add(menu);

        menuItem = new JMenuItem("Generate options...");
        menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.addActionListener(new ViewGenOptionsListener());
        menu.add(menuItem);

        // Create main panel.
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        setContentPane(mainPanel);

        m_view = new RangeView();
        m_view.addToolChangedListener(new RangeFrameToolChangedListener());
        mainPanel.add(m_view, BorderLayout.CENTER);

        // Create toolbar.
        m_toolBar = new JToolBar();
        Insets insets = new Insets(0, 0, 0, 0);
        AbstractButton button = null;
        URL buttonURL;
        mainPanel.add(m_toolBar, BorderLayout.NORTH);

        buttonURL = getClass().getResource("images/Point24.gif");
        button = new JToggleButton(new ImageIcon(buttonURL));
        button.setMargin(insets);
        button.addActionListener(new PointToolListener());
        m_toolBar.add(button);
        m_pointButton = (JToggleButton) button;

        buttonURL = getClass().getResource("images/Range24.gif");
        button = new JToggleButton(new ImageIcon(buttonURL));
        button.setMargin(insets);
        button.addActionListener(new RangeToolListener());
        m_toolBar.add(button);
        m_rangeButton = (JToggleButton) button;

        // Brute force button.
        button = new JButton("Brute force");
        button.setMargin(insets);
        button.addActionListener(new BruteForceListener());
        m_toolBar.add(button);
        
        // Range query button.
        button = new JButton("Range query");
        button.setMargin(insets);
        button.addActionListener(new RangeQueryListener());
        m_toolBar.add(button);
        
        // Generate button.
        button = new JButton("Generate");
        button.setMargin(insets);
        button.addActionListener(new GenerateListener());
        m_toolBar.add(button);
        
        // Set generation options.
        m_genPoints = 500;
        m_genDialog = new GenOptionsDialog(this);
        
        // Create file chooser.
        m_fileChooser = new JFileChooser();
        m_fileChooser.setCurrentDirectory(new File("."));
        m_fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        m_fileChooser.addChoosableFileFilter(new PointsFileFilter());

        // Init tool selection.
        m_view.setToolMode(RangeView.TOOL_POINT);

        pack();
    }

    class FileNewListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            /*GraphModel m = new GraphModel();
            m_view.setModel(m);*/
        }
    }

    class FileOpenListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            /*int retVal = m_fileChooser.showOpenDialog(MainFrame.this);
            if(retVal != JFileChooser.APPROVE_OPTION) {
                return;
            }

            String filename = m_fileChooser.getSelectedFile().getAbsolutePath();
            openFile(filename);*/
        }
    }

    protected void openFile(String filename) {
        /*try {
            FileInputStream is = new FileInputStream(filename);
            Reader r = new InputStreamReader(is);
            StreamTokenizer st = new StreamTokenizer(r);
            GraphModel m = new GraphModel();
            m.load(st);
            m_view.setModel(m);
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(this,
                                          ("Error opening file:\n" +
                                           ex.getMessage()),
                                          "Error opening file",
                                          JOptionPane.ERROR_MESSAGE);
        }*/
    }

    class FileSaveListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            /*int retVal = m_fileChooser.showSaveDialog(MainFrame.this);
            if(retVal != JFileChooser.APPROVE_OPTION) {
                return;
            }

            String filename = m_fileChooser.getSelectedFile().getAbsolutePath();
            saveFile(filename);*/
        }
    }

    protected void saveFile(String filename) {
        /*try {
            FileOutputStream os = new FileOutputStream(filename);
            Writer w = new OutputStreamWriter(os);
            GraphModel m = m_view.getModel();
            m.save(w);
            os.close();
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(this,
                                          "Error opening file",
                                          ("Error opening file:\n" +
                                           ex.getMessage()),
                                          JOptionPane.ERROR_MESSAGE);
        }*/
    }

    class FileExitListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            exit();
        }
    }

    class ViewGenOptionsListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            m_genDialog.initialize();
            m_genDialog.setVisible(true);
        }
    }

    class PointToolListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            m_view.setToolMode(RangeView.TOOL_POINT);
        }
    }

    class RangeToolListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            m_view.setToolMode(RangeView.TOOL_RANGE);
        }
    }

    class BruteForceListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            GeomTester2D tester = new GeomTester2DImpl();
            runQuery(new BruteForceRangeSearch2D(tester));
        }
    }
    
    class RangeQueryListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            GeomTester2D tester = new GeomTester2DImpl();
            runQuery(new RangeSearch2DImpl(tester));
        }
    }

    protected void runQuery(RangeSearch2D rs) {
        RangeModel model = m_view.getModel();
        if(model.getRange() == null) {
            m_view.setResultPoints(null);
            return;
        }

        rs.initialize(model.getPoints());
        Point2DIterator pi = rs.query(model.getRange());
        ArraySequence s = new ArraySequence();
        while(pi.hasNext()) {
            s.insertLast(pi.nextPoint2D());
        }
        m_view.setResultPoints(s);
    }
    
    class GenerateListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            m_view.getModel().generatePoints(m_genPoints, 50, 50, 350, 350);
        }
    }
    
    class RangeFrameToolChangedListener implements ToolChangedListener {
        public void toolChanged(ToolChangedEvent e) {
            boolean pointToolSelected = false;
            boolean rangeToolSelected = false;

            int toolMode = e.getToolMode();
            if(toolMode == RangeView.TOOL_POINT) {
                pointToolSelected   = true;
            } else if(toolMode == RangeView.TOOL_RANGE) {
                rangeToolSelected    = true;
            } else {
                // Nothing.
            }

            m_pointButton.setSelected(pointToolSelected);
            m_rangeButton.setSelected(rangeToolSelected);
        }
    }

    protected void exit() {
        System.exit(0);
    }

    class GenOptionsDialog extends JDialog {
        private JTextField m_genPointsField;
        public GenOptionsDialog(Frame frame) {
            super(frame, "Generator options", true);

            // Data fields.
            m_genPointsField = new JTextField("" + m_genPoints);
            
            // OK button.
            JButton okButton = new JButton("OK");
            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    m_genDialog.setVisible(false);
                    m_genPoints = Integer.parseInt(m_genPointsField.getText());
                }
            });
            getRootPane().setDefaultButton(okButton);

            // Cancel button.
            JButton cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    m_genDialog.setVisible(false);
                }
            });

            // Lay out data fields.
            JLabel label;
            JPanel dataPane = new JPanel();
            dataPane.setLayout(new BoxLayout(dataPane, BoxLayout.Y_AXIS));

            label = new JLabel("Number of points");
            label.setLabelFor(m_genPointsField);
            dataPane.add(label);
            dataPane.add(Box.createRigidArea(new Dimension(0, 5)));
            dataPane.add(m_genPointsField);
            dataPane.add(Box.createRigidArea(new Dimension(0, 10)));

            // Lay out buttons.
            JPanel buttonPane = new JPanel();
            buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
            Border border = BorderFactory.createEmptyBorder(0, 10, 10, 10);
            buttonPane.setBorder(border);
            buttonPane.add(Box.createHorizontalGlue());
            buttonPane.add(cancelButton);
            buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
            buttonPane.add(okButton);

            // Lay out panel.
            java.awt.Container contentPane = getContentPane();
            contentPane.add(dataPane  , BorderLayout.CENTER);
            contentPane.add(buttonPane, BorderLayout.SOUTH);
            pack();
        }

        public void initialize() {
            m_genPointsField.setText("" + m_genPoints);
        }
    }
}

class PointsFileFilter extends FileFilter {
    public boolean accept(File f) {
        if(f.isDirectory()) {
            return true;
        }
        String ext = getExtension(f);
        if(ext != null && ext.equals("pts")) {
            return true;
        }
        return false;
    }

    public String getDescription() {
        return "Point files (*.pts)";
    }

    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if(i > 0 && i < s.length() - 1) {
            ext = s.substring(i + 1).toLowerCase();
        }
        return ext;
    }
}


