package rangesearch.ref;

import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;

/** A comparator to compare points by y coordinate.  Horizontally aligned points
 * are considered equal. */

public class AboveBelowComparator extends AbstractComparator implements Comparator {
    protected GeomTester2D m_tester;

    /** @param tester the geometry tester to use for comparisions. */
    public AboveBelowComparator(GeomTester2D tester) {
        m_tester = tester;
    }

    /** @param a the first object to compare.
     * @param b the second object to compare.
     * @throws InvalidPointException if the point comparision fails. */
    public int compare(Object a, Object b)
        throws ClassCastException {
        Point2D pa = (Point2D) a;
        Point2D pb = (Point2D) b;
        int test = m_tester.aboveBelow(pa, pb);
        if(test == GeomTester2D.BELOW) { 
            return -1;
        } else if(test == GeomTester2D.ABOVE) {
            return  1;
        } else {
            return  0;
        }
    }

    /** Returns whether or not an object is comparable with this comparator.
     * @param a the object to consider.
     * @return true if the object is a point, false otherwise.
     */
    public boolean isComparable(Object a) {
        if(a == null) return false;
        try {
            Point2D pa = (Point2D) a;
            return true;
        } catch (ClassCastException ex) {
            return false;
        }
    }
}
