package rangesearch.ref;

import rangesearch.api.*;

import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;

/** This class searches a 2D set of points within a rectangle in a brute force
 * manner. */

public class BruteForceRangeSearch2D implements RangeSearch2D {
    protected Sequence m_points;
    protected GeomTester2D m_tester;

    /** @param tester the geometry tester to use for comparisions. */
    public BruteForceRangeSearch2D(GeomTester2D tester) {
        m_points = null;
        m_tester = tester;
    }

    /** Initializes the range searcher with the given set of points.
     */
    public void initialize(Point2DIterator pi) {
        m_points = new ArraySequence();
        while(pi.hasNext()) {
            m_points.insertLast(pi.nextPoint2D());
        }
    }

    /** Queries the current set of points using the query rectangle r.
     */
    public Point2DIterator query(Rectangle2D r) {
        Assertion.check(m_points != null, "query called before initialize.");
        Assertion.check(r != null, "null query range");

        // Go through the list of points, check each one to see if it is within
        // the given rectangle.
        Sequence s = new ArraySequence();
        Point2D p1 = r.topLeftPoint();
        Point2D p2 = r.bottomRightPoint();
        Point2DIterator pi = new Point2DIteratorAdapter(m_points.elements());
        while(pi.hasNext()) {
            Point2D p = pi.nextPoint2D();
            int test1 = m_tester.leftRight(p1, p);
            int test2 = m_tester.leftRight(p, p2);
            if((test1 == GeomTester2D.LEFT ||
                test1 == GeomTester2D.ON) &&
               (test2 == GeomTester2D.LEFT ||
                test2 == GeomTester2D.ON)) {
                int test3 = m_tester.aboveBelow(p1, p);
                int test4 = m_tester.aboveBelow(p, p2);
                if((test3 == GeomTester2D.ABOVE ||
                    test3 == GeomTester2D.ON) &&
                   (test4 == GeomTester2D.ABOVE ||
                    test4 == GeomTester2D.ON)) {
                    s.insertLast(p);
                }
            }
        }
        return new Point2DIteratorAdapter(s.elements());
    }
}


