package rangesearch.ref;

import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;

/** A comparator to compare points by x coordinate.  Vertically aligned points are
 * considered equal. */

public class LeftRightComparator extends AbstractComparator implements Comparator {
    protected GeomTester2D m_tester;

    /** @param tester the geometry tester to use for comparisions. */
    public LeftRightComparator(GeomTester2D tester) {
        m_tester = tester;
    }

    /** @param a the first object to compare.
     * @param b the second object to compare.
     * @throws InvalidPointException if the point comparision fails. */
    public int compare(Object a, Object b)
        throws ClassCastException {
        Point2D pa = (Point2D) a;
        Point2D pb = (Point2D) b;
        int test = m_tester.leftRight(pa, pb);
        if(test == GeomTester2D.LEFT) { 
            return -1;
        } else if(test == GeomTester2D.RIGHT) {
            return  1;
        } else {
            return  0;
        }
    }

    /** Returns whether or not an object is comparable with this comparator.
     * @param a the object to consider.
     * @return true if the object is a point, false otherwise.
     */
    public boolean isComparable(Object a) {
        try {
            Point2D pa = (Point2D) a;
            return true;
        } catch (ClassCastException ex) {
            return false;
        }
    }
}


