package rangesearch.ref;

import rangesearch.api.*;

import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;

/** An implementation of the RangeSearch2D structure using a range tree. */

public class RangeSearch2DImpl implements RangeSearch2D {
    protected RangeTree2D m_rangeTree;
    protected GeomTester2D m_tester;

    /** @param tester the geometric tester to use for point/rectangle
     *                comparisions.
     */
    public RangeSearch2DImpl(GeomTester2D tester) {
        m_rangeTree = null;
        m_tester = tester;
    }

    /** Initializes the range searcher with the given set of points.
     * @param pi an iterator over the points to be searched.
     * @throws RangeSearchException if points can't be compared properly.
     */
    public void initialize(Point2DIterator pi) 
            throws RangeSearchException {
        Comparator lrComparator = new LeftRightComparator(m_tester);
        Comparator abComparator = new AboveBelowComparator(m_tester);
        m_rangeTree = new RangeTree2DImpl();

        try {
            m_rangeTree.initialize(pi, lrComparator, abComparator);
        } catch (GeomException ex) {
            // In the case of geometric test failure.
            throw new RangeSearchException(ex.getMessage());
        }
    }

    /** Queries the current set of points using the query rectangle r.
     * Points are considered to be within the query rectangle if they lie on or
     * within the rectangle's borders.
     * @param r the rectangle in which to query.
     * @throws RangeSearchException if points can't be compared properly.
     */
    public Point2DIterator query(Rectangle2D r)
            throws RangeSearchException {
        Assertion.check(m_rangeTree != null, "query called before initialize.");
        Assertion.check(r != null, "null query range");

        Point2D p1 = r.bottomLeftPoint();
        Point2D p2 = r.topRightPoint();

        ObjectIterator oi;
        try {
            oi = m_rangeTree.query(p1, p2, p1, p2);
        } catch (GeomException ex) {
            // In the case of geometric test failure.
            throw new RangeSearchException(ex.getMessage());
        }

        return (Point2DIterator) new Point2DIteratorAdapter(oi);
    }
}

