package rangesearch.test;

import rangesearch.api.*;
import rangesearch.ref.*;
import rangesearch.*;

import jdsl.core.api.*;
import jdsl.core.ref.*;
import jdsl.geomobj.api.*;
import jdsl.geomobj.ref.*;
import junit.framework.TestCase;

/** Simple utilities used only in testing. */

class RangeSearch2DUtil extends TestCase {
    public RangeSearch2DUtil(String name) {
        super(name);
    }

    /** A method to compare the output of a range search with the expected.
	@param rs an instance of the rangesearch
	@param r a query rectangle
	@param s a string containing the properly formatted result that is
	expeceted.
    */
    protected void checkQuery(RangeSearch2D rs, Rectangle2D r, String s) {
        Point2DIterator pi = rs.query(r);
        assertEquals(s, JDSLUtil.pointListToString(pi));
    }

    /** Converts a sequence of points and returns them in the iterator used 
	within the Range* classes 
	@param coords an array with twice the entries 
	as the number of points of the form [x1,y1,x2,y2,...]
	@returns an iterator over IntPoint2D's
    */
    protected Point2DIterator buildPointSequence(int[] coords) {
        Assertion.check(coords.length % 2 == 0);
        Sequence s = new ArraySequence();
        for(int i = 0; i < coords.length / 2; i ++) {
            s.insertLast(new IntPoint2D(coords[i*2 + 0],
                                        coords[i*2 + 1]));
        }
        return (Point2DIterator) new Point2DIteratorAdapter(s.elements());
    }

    /** A method to create a rectangle (for a query, ie.)
	@param x1 the x coordinate of the lower lefthand corner of the 
	query rectangle
	@param y1 the y coordinate of the lower lefthand corner of the 
	query rectangle
	@param x2 the x coordinate of the upper righthand corner of the 
	query rectangle
	@param y2 the y coordinate of the upper righthand corner of the 
	query rectangle
	@returns the Rectangle2D represented by these points
    */
    protected Rectangle2D buildRect(int x1, int y1, int x2, int y2) {
        return new IntRectangle2D(new IntPoint2D(x1, y1),
                                  new IntPoint2D(x2, y2));
    }
}

